-- Nancy Minderman 
-- nancy.minderman@ualberta.ca
-- This file makes extensive use of Altera template structures.
-- This file is the top-level file for lab 1 winter 2014 for version 12.1sp1 on Windows 7
--
-- Modified by Hang Peng, Ryan Corpuz, JingjingLiang 
-- This file is the toplevel of Coloured-object tracking camera.


-- A library clause declares a name as a library.  It 
-- does not create the library; it simply forward declares 
-- it. 
library ieee;

-- Commonly imported packages:

	-- STD_LOGIC and STD_LOGIC_VECTOR types, and relevant functions
	use ieee.std_logic_1164.all;

	-- SIGNED and UNSIGNED types, and relevant functions
	use ieee.numeric_std.all;

	-- Basic sequential functions and concurrent procedures
	use ieee.VITAL_Primitives.all;
	
	use work.DE2_CONSTANTS.all;
	
entity niosII_microc_lab1 is
	
	port
	(
		-- Input ports and 50 MHz Clock
		KEY			: in  	std_logic_vector (0 downto 0);
		SW			: in 	std_logic_vector (17 downto 0);
		CLOCK_50	: in  	std_logic;
		GPIO_0		:OUT 	std_logic_vector (1 downto 0);
		
		-- Green leds on board
		LEDG		: out DE2_LED_GREEN;
		
		-- LCD on board
		LCD_BLON	: out 	std_logic;
		LCD_ON		: out 	std_logic;
		LCD_DATA	: inout DE2_LCD_DATA_BUS;
		LCD_RS		: out 	std_logic;
		LCD_EN		: out 	std_logic;
		LCD_RW		: out 	std_logic;
		
		-- SDRAM on board

		DRAM_ADDR	:	out		DE2_SDRAM_ADDR_BUS;
		DRAM_DQ		:	inout 	DE2_SDRAM_DATA_BUS;
		DRAM_BA_0	: 	out		std_logic;
		DRAM_BA_1	:	out		std_logic;
		DRAM_CAS_N	:	out		std_logic;
		DRAM_CKE	:	out		std_logic;
		DRAM_CLK	:	out		std_logic;
		DRAM_CS_N	:	out		std_logic;
		DRAM_LDQM	: 	out		std_logic;
		DRAM_UDQM	: 	out		std_logic;
		DRAM_RAS_N	: 	out		std_logic;
		DRAM_WE_N	: 	out 	std_logic;

		-- SRAM on board
		
		SRAM_ADDR	:	out		DE2_SRAM_ADDR_BUS;
		SRAM_DQ		:	inout 	DE2_SRAM_DATA_BUS;
		SRAM_WE_N	:	out		std_logic;
		SRAM_OE_N	:	out		std_logic;
		SRAM_UB_N	:	out 	std_logic;
		SRAM_LB_N	:	out		std_logic;
		SRAM_CE_N	:	out		std_logic;
		
		-- VGA Controller
		VGA_R 		: 	OUT 	STD_LOGIC_VECTOR (9 downto 0);
		VGA_B 		: 	OUT 	STD_LOGIC_VECTOR (9 downto 0);
		VGA_G 		: 	OUT 	STD_LOGIC_VECTOR (9 downto 0);
		VGA_CLK		: 	OUT 	STD_LOGIC;
		VGA_BLANK	: 	OUT 	STD_LOGIC;
		VGA_HS		:	OUT 	STD_LOGIC;
		VGA_VS		: 	OUT 	STD_LOGIC;
		VGA_SYNC	: 	OUT 	STD_LOGIC;
		
		-- TV decoder
		I2C_SDAT    : 	inout 	std_logic                   := 'X';             -- SDAT
		I2C_SCLK    :	out   	std_logic;
		
		TD_CLK27    : 	in    std_logic                     := 'X';             -- TD_CLK27
		TD_DATA     : 	in    std_logic_vector(7 downto 0)  := (others => 'X'); -- TD_DATA
		TD_HS       : 	in    std_logic                     := 'X';             -- TD_HS
		TD_VS       : 	in    std_logic                     := 'X';             -- TD_VS
		TD_RESET    :	out   std_logic;                                        -- TD_RESET
	
		-- Flash memory
		FL_ADDR		: 	out 	std_LOGIC_VECTOR(21 downto 0);
		FL_CE_N		: 	out 	std_LOGIC_VECTOR(0 downto 0);
		FL_OE_N		: 	out 	STD_LOGIC_VECTOR(0 downto 0);
		FL_DQ		: 	inout 	std_LOGIC_VECTOR(7 downto 0);
		FL_RST_N	: 	out 	std_LOGIC_VECTOR(0 downto 0) := (others =>'1');
		FL_WE_N		: 	out 	std_LOGIC_VECTOR(0 downto 0)
		
		);
end niosII_microc_lab1;


architecture structure of niosII_microc_lab1 is

	-- Declarations (optional)
	
	  component video_sys is
        port (
			square_switch_external_connection_export		: IN std_logic;
			servo_0_conduit_end_0_export						 :OUT std_logic;
			servo_1_conduit_end_0_export						 :OUT std_logic;
			image_processor_0_conduit_end_export    : IN std_logic;
            VGA_CLK_from_the_VGA_Controller         : out   std_logic;                                        -- CLK
            VGA_HS_from_the_VGA_Controller          : out   std_logic;                                        -- HS
            VGA_VS_from_the_VGA_Controller          : out   std_logic;                                        -- VS
            VGA_BLANK_from_the_VGA_Controller       : out   std_logic;                                        -- BLANK
            VGA_SYNC_from_the_VGA_Controller        : out   std_logic;                                        -- SYNC
            VGA_R_from_the_VGA_Controller           : out   std_logic_vector(9 downto 0);                     -- R
            VGA_G_from_the_VGA_Controller           : out   std_logic_vector(9 downto 0);                     -- G
            VGA_B_from_the_VGA_Controller           : out   std_logic_vector(9 downto 0);                     -- B
            clk_0                                   : in    std_logic                     := 'X';             -- clk
            reset_n                                 : in    std_logic                     := 'X';             -- reset_n
            I2C_SDAT_to_and_from_the_AV_Config      : inout std_logic                     := 'X';             -- SDAT
            I2C_SCLK_from_the_AV_Config             : out   std_logic;                                        -- SCLK
            SRAM_DQ_to_and_from_the_Pixel_Buffer    : inout DE2_SRAM_DATA_BUS := (others => 'X'); -- DQ
            SRAM_ADDR_from_the_Pixel_Buffer         : out DE2_SRAM_ADDR_BUS;                    -- ADDR
            SRAM_LB_N_from_the_Pixel_Buffer         : out   std_logic;                                        -- LB_N
            SRAM_UB_N_from_the_Pixel_Buffer         : out   std_logic;                                        -- UB_N
            SRAM_CE_N_from_the_Pixel_Buffer         : out   std_logic;                                        -- CE_N
            SRAM_OE_N_from_the_Pixel_Buffer         : out   std_logic;                                        -- OE_N
            SRAM_WE_N_from_the_Pixel_Buffer         : out   std_logic;                                        -- WE_N
            TD_CLK27_to_the_Video_In_Decoder        : in    std_logic                     := 'X';             -- TD_CLK27
            TD_DATA_to_the_Video_In_Decoder         : in    std_logic_vector(7 downto 0)  := (others => 'X'); -- TD_DATA
            TD_HS_to_the_Video_In_Decoder           : in    std_logic                     := 'X';             -- TD_HS
            TD_VS_to_the_Video_In_Decoder           : in    std_logic                     := 'X';             -- TD_VS
            TD_RESET_from_the_Video_In_Decoder      : out   std_logic;                                      -- TD_RESET
			
			sdram_0_wire_addr                       : out   DE2_SDRAM_ADDR_BUS;                    -- addr
            sdram_0_wire_ba                         : out   std_logic_vector(1 downto 0);                     -- ba
            sdram_0_wire_cas_n                      : out   std_logic;                                        -- cas_n
            sdram_0_wire_cke                        : out   std_logic;                                        -- cke
            sdram_0_wire_cs_n                       : out   std_logic;                                        -- cs_n
            sdram_0_wire_dq                         : inout DE2_SDRAM_DATA_BUS := (others => 'X'); -- dq
            sdram_0_wire_dqm                        : out   std_logic_vector(1 downto 0);                     -- dqm
            sdram_0_wire_ras_n                      : out   std_logic;                                        -- ras_n
            sdram_0_wire_we_n                       : out   std_logic;
			clock_signals_sdram_clk_clk             : out   std_logic;
			led_external_connection_export          : out   DE2_LED_GREEN;
				
			character_lcd_external_interface_DATA 	: inout DE2_LCD_DATA_BUS := (OTHERS => 'X');
			character_lcd_external_interface_ON		: out   std_logic;
			character_lcd_external_interface_BLON	: out   std_logic;
			character_lcd_external_interface_EN		: out   std_logic;
			character_lcd_external_interface_RS		: out   std_logic;
			character_lcd_external_interface_RW		: out   std_logic;
				
			config_switch_external_export				: in	std_logic := 'X';
			h_lock_switch_external_export				: in	std_logic := 'X';
			v_lock_switch_external_export				: in	std_logic := 'X';
				
			tristate_conduit_bridge_0_out_generic_tristate_controller_0_tcm_read_n_out       : out   std_logic_vector(0 downto 0);                     -- generic_tristate_controller_0_tcm_read_n_out
            tristate_conduit_bridge_0_out_generic_tristate_controller_0_tcm_data_out         : inout std_logic_vector(7 downto 0)  := (others => 'X'); -- generic_tristate_controller_0_tcm_data_out
            tristate_conduit_bridge_0_out_generic_tristate_controller_0_tcm_chipselect_n_out : out   std_logic_vector(0 downto 0);                     -- generic_tristate_controller_0_tcm_chipselect_n_out
            tristate_conduit_bridge_0_out_generic_tristate_controller_0_tcm_write_n_out      : out   std_logic_vector(0 downto 0);                     -- generic_tristate_controller_0_tcm_write_n_out
            tristate_conduit_bridge_0_out_generic_tristate_controller_0_tcm_address_out      : out   std_logic_vector(21 downto 0)
        );
    end component video_sys;

    

--	These signals are for matching the provided IP core to
-- The specific SDRAM chip in our system	 
	 signal BA	: std_logic_vector (1 downto 0);
	 signal DQM	:	std_logic_vector (1 downto 0);
	 

begin

	DRAM_BA_1 <= BA(1);
	DRAM_BA_0 <= BA(0);
	
	DRAM_UDQM <= DQM(1);
	DRAM_LDQM <= DQM(0);
	
	-- Component Instantiation Statement (optional)
	
	  u0 : component video_sys
        port map (
			servo_0_conduit_end_0_export						 => GPIO_0(0),
			servo_1_conduit_end_0_export						 => GPIO_0(1),
				
            VGA_CLK_from_the_VGA_Controller         => VGA_CLK,         --   VGA_Controller_external_interface.CLK
            VGA_HS_from_the_VGA_Controller          => VGA_HS,          --                                    .HS
            VGA_VS_from_the_VGA_Controller          => VGA_VS,          --                                    .VS
            VGA_BLANK_from_the_VGA_Controller       => VGA_BLANK,       --                                    .BLANK
            VGA_SYNC_from_the_VGA_Controller        => VGA_SYNC,        --                                    .SYNC
            VGA_R_from_the_VGA_Controller           => VGA_R,           --                                    .R
            VGA_G_from_the_VGA_Controller           => VGA_G,           --                                    .G
            VGA_B_from_the_VGA_Controller           => VGA_B,           --                                    .B
            clk_0                                   => CLOCK_50,                                   --                        clk_0_clk_in.clk
            reset_n                                 => KEY(0),                                 --                  clk_0_clk_in_reset.reset_n
            I2C_SDAT_to_and_from_the_AV_Config      => I2C_SDAT,      --        AV_Config_external_interface.SDAT
            I2C_SCLK_from_the_AV_Config             => I2C_SCLK,             --                                    .SCLK
            SRAM_DQ_to_and_from_the_Pixel_Buffer    => SRAM_DQ,    --     Pixel_Buffer_external_interface.DQ
            SRAM_ADDR_from_the_Pixel_Buffer         => SRAM_ADDR,         --                                    .ADDR
            SRAM_LB_N_from_the_Pixel_Buffer         => SRAM_LB_N,         --                                    .LB_N
            SRAM_UB_N_from_the_Pixel_Buffer         => SRAM_UB_N,         --                                    .UB_N
            SRAM_CE_N_from_the_Pixel_Buffer         => SRAM_CE_N,         --                                    .CE_N
            SRAM_OE_N_from_the_Pixel_Buffer         => SRAM_OE_N,         --                                    .OE_N
            SRAM_WE_N_from_the_Pixel_Buffer         => SRAM_WE_N,         --                                    .WE_N
            TD_CLK27_to_the_Video_In_Decoder        => TD_CLK27,        -- Video_In_Decoder_external_interface.TD_CLK27
            TD_DATA_to_the_Video_In_Decoder         => TD_DATA,         --                                    .TD_DATA
            TD_HS_to_the_Video_In_Decoder           => TD_HS,           --                                    .TD_HS
            TD_VS_to_the_Video_In_Decoder           => TD_VS,           --                                    .TD_VS
            TD_RESET_from_the_Video_In_Decoder      => TD_RESET,     --                                    .TD_RESET
           
			sdram_0_wire_addr                       => DRAM_ADDR,                       --                        sdram_0_wire.addr
            sdram_0_wire_ba                         => BA,                         --                                    .ba
            sdram_0_wire_cas_n                      => DRAM_CAS_N,                      --                                    .cas_n
            sdram_0_wire_cke                        => DRAM_CKE,                        --                                    .cke
            sdram_0_wire_cs_n                       => DRAM_CS_N,                       --                                    .cs_n
            sdram_0_wire_dq                         => DRAM_DQ,                         --                                    .dq
            sdram_0_wire_dqm                        => DQM,                        --                                    .dqm
            sdram_0_wire_ras_n                      => DRAM_RAS_N,                      --                                    .ras_n
            sdram_0_wire_we_n                       => DRAM_WE_N,
			clock_signals_sdram_clk_clk             => DRAM_CLK,
			led_external_connection_export			 => LEDG,
				
			tristate_conduit_bridge_0_out_generic_tristate_controller_0_tcm_read_n_out => FL_OE_N,
			tristate_conduit_bridge_0_out_generic_tristate_controller_0_tcm_data_out => FL_DQ,
			tristate_conduit_bridge_0_out_generic_tristate_controller_0_tcm_address_out => FL_ADDR,
			tristate_conduit_bridge_0_out_generic_tristate_controller_0_tcm_write_n_out => FL_WE_N,
			tristate_conduit_bridge_0_out_generic_tristate_controller_0_tcm_chipselect_n_out => FL_CE_N,
				
			--Switches
			config_switch_external_export	=> SW(0),
			h_lock_switch_external_export	=> SW(17),
			v_lock_switch_external_export	=> SW(16),
			image_processor_0_conduit_end_export => SW(1),
			square_switch_external_connection_export => SW(2);
			--LCD
			character_lcd_external_interface_DATA 	=> LCD_DATA,
			character_lcd_external_interface_ON		=> LCD_ON,
			character_lcd_external_interface_BLON	=> LCD_BLON,
			character_lcd_external_interface_EN		=> LCD_EN,
			character_lcd_external_interface_RS		=> LCD_RS,
			character_lcd_external_interface_RW		=> LCD_RW	
        );

end structure;


library ieee;

-- Commonly imported packages:

	-- STD_LOGIC and STD_LOGIC_VECTOR types, and relevant functions
	use ieee.std_logic_1164.all;

package DE2_CONSTANTS is
	
	type DE2_SDRAM_ADDR_BUS is array(11 downto 0) of std_logic;
	type DE2_SDRAM_DATA_BUS is array(15 downto 0) of std_logic;
	
	type DE2_LCD_DATA_BUS	is array(7 downto 0) of std_logic;
	
	type DE2_LED_GREEN		is array(7 downto 0)  of std_logic;
	
	type DE2_SRAM_ADDR_BUS	is array(17 downto 0) of std_logic;
	type DE2_SRAM_DATA_BUS  is array(15 downto 0) of std_logic;
	
end DE2_CONSTANTS;



