LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_unsigned.all;
USE ieee.std_logic_misc.all;

-- *****************************************************************************
-- * This file is the top-level file of image processor.                       *
-- *****************************************************************************

ENTITY rgb_to_hsv_conversion_and_threshold IS 

-- *****************************************************************************
-- *                             Generic Declarations                          *
-- *****************************************************************************
	
GENERIC (
	
	IW		:INTEGER									:= 23;
	OW		:INTEGER									:= 1;
	
	EW		:INTEGER									:=0;
	
	EIW		:INTEGER									:= 1;
	EOW		:INTEGER									:= 1
	
);
-- *****************************************************************************
-- *                             Port Declarations                             *
-- *****************************************************************************
PORT (

	-- Inputs
	clk								:IN		STD_LOGIC;
	reset							:IN		STD_LOGIC;

	stream_in_data					:IN		STD_LOGIC_VECTOR(IW DOWNTO  0);	
	stream_in_startofpacket			:IN		STD_LOGIC;
	stream_in_endofpacket			:IN		STD_LOGIC;
	stream_in_valid					:IN		STD_LOGIC;

	stream_out_ready				:IN		STD_LOGIC;
	show_thresh						:IN		STD_LOGIC;
	
	avs_H_write						:IN 	STD_LOGIC;
	avs_S_write						:IN 	STD_LOGIC;
	avs_V_write						:IN 	STD_LOGIC;

	avs_H_writedata 				:IN 	STD_LOGIC_VECTOR (31 downto 0);
	avs_S_writedata 				:IN 	STD_LOGIC_VECTOR (31 downto 0);
	avs_V_writedata 				:IN 	STD_LOGIC_VECTOR (31 downto 0);

	avs_H_read						:IN 	STD_LOGIC;
	avs_S_read						:IN 	STD_LOGIC;
	avs_V_read						:IN 	STD_LOGIC;

	avs_H_readdata 					:OUT 	STD_LOGIC_VECTOR (31 downto 0);
	avs_S_readdata 					:OUT 	STD_LOGIC_VECTOR (31 downto 0);
	avs_V_readdata 					:OUT 	STD_LOGIC_VECTOR (31 downto 0);
	
	avs_s1_read						:IN 	STD_LOGIC;
	avs_s2_read						:IN 	STD_LOGIC;
	avs_s3_read						:IN 	STD_LOGIC;
	avs_s4_read						:IN 	STD_LOGIC;
	avs_s5_read						:IN 	STD_LOGIC;
	
	avs_00_read 	: IN STD_LOGIC;
	avs_01_read	 	: IN STD_LOGIC;
	avs_02_read 	: IN STD_LOGIC;
	avs_10_read 	: IN STD_LOGIC;
	avs_11_read 	: IN STD_LOGIC;
	avs_12_read 	: IN STD_LOGIC;
	avs_20_read 	: IN STD_LOGIC;
	avs_21_read 	: IN STD_LOGIC;
	avs_22_read 	: IN STD_LOGIC;
	avs_range_write : IN STD_LOGIC;
	avs_range_writedata		:IN STD_LOGIC_vector(31 DOWNTO 0);

	-- Bidirectional

	-- Outputs
	stream_in_ready				:BUFFER	STD_LOGIC;
	stream_out_data				:BUFFER	STD_LOGIC_VECTOR(IW DOWNTO 0);	
	stream_out_startofpacket	:BUFFER	STD_LOGIC;
	stream_out_endofpacket		:BUFFER	STD_LOGIC;
	stream_out_valid			:BUFFER	STD_LOGIC;
	
	avs_s1_readdata : OUT STD_LOGIC_VECTOR (31 downto 0);
	avs_s2_readdata : OUT STD_LOGIC_VECTOR (31 downto 0);
	avs_s3_readdata : OUT STD_LOGIC_VECTOR (31 downto 0);
	avs_s4_readdata : OUT STD_LOGIC_VECTOR (31 downto 0);
	avs_s5_readdata : OUT STD_LOGIC_VECTOR (31 downto 0);
	
	
	
	avs_00_readdata : OUT STD_LOGIC_VECTOR (31 downto 0);
	avs_01_readdata : OUT STD_LOGIC_VECTOR (31 downto 0);
	avs_02_readdata : OUT STD_LOGIC_VECTOR (31 downto 0);
	avs_10_readdata : OUT STD_LOGIC_VECTOR (31 downto 0);
	avs_11_readdata : OUT STD_LOGIC_VECTOR (31 downto 0);
	avs_12_readdata : OUT STD_LOGIC_VECTOR (31 downto 0);
	avs_20_readdata : OUT STD_LOGIC_VECTOR (31 downto 0);
	avs_21_readdata : OUT STD_LOGIC_VECTOR (31 downto 0);
	avs_22_readdata : OUT STD_LOGIC_VECTOR (31 downto 0)
	

);

END rgb_to_hsv_conversion_and_threshold;

ARCHITECTURE Behaviour OF rgb_to_hsv_conversion_and_threshold IS
-- *****************************************************************************
-- *                       Internal Signals Declarations                       *
-- *****************************************************************************
	
	-- Internal Wires
	SIGNAL	transfer_data					:STD_LOGIC;
		
	SIGNAL	converted_data					:STD_LOGIC_VECTOR(IW DOWNTO  0);
	SIGNAL	converted_startofpacket			:STD_LOGIC;
	SIGNAL	converted_endofpacket			:STD_LOGIC;
	SIGNAL	converted_valid					:STD_LOGIC;
	
	-- Internal Registers
	SIGNAL	data							:STD_LOGIC_VECTOR(IW DOWNTO  0);	
	SIGNAL	startofpacket					:STD_LOGIC;
	SIGNAL	endofpacket						:STD_LOGIC;
	SIGNAL	valid							:STD_LOGIC;
	
	SIGNAL 	H_thresh						:std_logic_vector (31 downto 0);
	SIGNAL 	V_thresh						:std_logic_vector (31 downto 0);
	SIGNAL	S_thresh						:std_logic_vector (31 downto 0);
	

		
	-- 9 bits for hue (0-360) -- 7 bits for saturation (0 to 100) -- 7 bits for value (0 to 100)
	SIGNAL threshold_data_in					:std_logic_vector (31 downto 0);
	
	SIGNAL location							:std_logic_vector (31 downto 0);
	SIGNAL temp1_result						:std_logic_vector (31 downto 0);
	SIGNAL temp2_result						:std_logic_vector (31 downto 0);
	SIGNAL temp3_result						:std_logic_vector (31 downto 0);
	SIGNAL temp4_result						:std_logic_vector (31 downto 0);
	
	SIGNAL pixel_00							:std_lOGIC_vector (31 downto 0);
	SIGNAL pixel_01							:std_lOGIC_vector (31 downto 0);
	SIGNAL pixel_02							:std_lOGIC_vector (31 downto 0);
	SIGNAL pixel_10							:std_lOGIC_vector (31 downto 0);
	SIGNAL pixel_11							:std_lOGIC_vector (31 downto 0);
	SIGNAL pixel_12							:std_lOGIC_vector (31 downto 0);
	SIGNAL pixel_20							:std_lOGIC_vector (31 downto 0);
	SIGNAL pixel_21							:std_lOGIC_vector (31 downto 0);
	SIGNAL pixel_22							:std_lOGIC_vector (31 downto 0);
	SIGNAL threshold_range 				   	:std_lOGIC_vector (31 downto 0);
	SIGNAL pass_val							:std_logic;


	COMPONENT RGB_to_HSV_converter
	PORT (
		-- Inputs
		clk									:IN		STD_LOGIC;
		clk_en								:IN		STD_LOGIC;
		reset								:IN		STD_LOGIC;

		R									:IN		STD_LOGIC_VECTOR( 7 DOWNTO  0);
		G									:IN		STD_LOGIC_VECTOR( 7 DOWNTO  0);
		B									:IN		STD_LOGIC_VECTOR( 7 DOWNTO  0);

		H									:IN		STD_LOGIC_VECTOR( 8 DOWNTO  0);
		S									:IN		STD_LOGIC_VECTOR( 7 DOWNTO  0);
		V									:IN		STD_LOGIC_VECTOR( 7 DOWNTO  0);
		
		H_range								:IN		STD_LOGIC_VECTOR( 7 DOWNTO  0);
		S_range								:IN		STD_LOGIC_VECTOR( 7 DOWNTO  0);
		V_range								:IN		STD_LOGIC_VECTOR( 7 DOWNTO  0);
		
		stream_in_startofpacket				:IN		STD_LOGIC;
		thershould_range					:IN 	STD_LOGIC_vector(31 DOWNTO 0);

		-- Outputs
		location_x							:OUT STD_LOGIC_VECTOR(9 DOWNTO 0);
		location_y							:OUT STD_LOGIC_VECTOR(9 DOWNTO 0);
		
		temp1								:OUT STD_LOGIC_vector(31 DOWNTO 0);
		temp2								:OUT STD_LOGIC_vector(31 DOWNTO 0);
		temp3								:OUT STD_LOGIC_vector(31 DOWNTO 0);
		temp4								:OUT STD_LOGIC_vector(31 DOWNTO 0);
		
		stream_out_thresh					:BUFFER  STD_LOGIC;
		
		sample_pixel_00						:OUT STD_LOGIC_vector(31 DOWNTO 0);
		sample_pixel_10						:OUT STD_LOGIC_vector(31 DOWNTO 0);
		sample_pixel_20						:OUT STD_LOGIC_vector(31 DOWNTO 0);
		sample_pixel_01						:OUT STD_LOGIC_vector(31 DOWNTO 0);
		sample_pixel_11						:OUT STD_LOGIC_vector(31 DOWNTO 0);
		sample_pixel_21						:OUT STD_LOGIC_vector(31 DOWNTO 0);
		sample_pixel_02						:OUT STD_LOGIC_vector(31 DOWNTO 0);
		sample_pixel_12						:OUT STD_LOGIC_vector(31 DOWNTO 0);
		sample_pixel_22						:OUT STD_LOGIC_vector(31 DOWNTO 0)
		
	);
	END COMPONENT;
BEGIN
-- *****************************************************************************
-- *                         Finite State Machine(s)                           *
-- *****************************************************************************


-- *****************************************************************************
-- *                             Sequential Logic                              *
-- *****************************************************************************

latch_H_value: process(clk,avs_H_write) is
	begin
		if falling_edge(avs_H_write) then
			H_thresh <= avs_H_writedata;
		end if;
	end process;
	
write_H_value: process(clk,avs_H_read) is
	begin
		if rising_edge(avs_H_read) then
			avs_H_readdata <= H_thresh;
		end if;
	end process;
	
latch_S_value: process(clk,avs_S_write) is
	begin
		if falling_edge(avs_S_write) then
			S_thresh <= avs_S_writedata;
		end if;
	end process;
	
write_S_value: process(clk,avs_S_read) is
	begin
		if rising_edge(avs_S_read) then
			avs_S_readdata <= S_thresh;
		end if;
	end process;
	
latch_V_value: process(clk,avs_V_write) is
	begin
		if falling_edge(avs_V_write) then
			V_thresh <= avs_V_writedata;
		end if;
	end process;

write_V_value: process(clk,avs_V_read) is
	begin
		if rising_edge(avs_V_read) then
			avs_V_readdata <= V_thresh;
		end if;
	end process;
	
write_location: process(clk,avs_s1_read) is
	begin
		if rising_edge(avs_s1_read) then
			avs_s1_readdata <= location;
		end if;
	end process;
	
write_temp1: process(clk,avs_s2_read) is
	begin
		if rising_edge(avs_s2_read) then
			avs_s2_readdata <= temp1_result;
		end if;
	end process;

write_temp2: process(clk,avs_s3_read) is
	begin
		if rising_edge(avs_s3_read) then
			avs_s3_readdata <= temp2_result;
		end if;
	end process;

write_temp3: process(clk,avs_s4_read) is
	begin
		if rising_edge(avs_s4_read) then
			avs_s4_readdata <= temp3_result;
		end if;
	end process;
	
write_temp4: process(clk,avs_s5_read) is
	begin
		if rising_edge(avs_s5_read) then
			avs_s5_readdata <= temp4_result;
		end if;
	end process;
	
read_00: process(clk,avs_00_read) is
	begin
		if rising_edge(avs_00_read) then
			avs_00_readdata <= pixel_00;
		end if;
	end process;
	
read_01: process(clk,avs_01_read) is
	begin
		if rising_edge(avs_01_read) then
			avs_01_readdata <= pixel_01;
		end if;
	end process;
	
read_02: process(clk,avs_02_read) is
	begin
		if rising_edge(avs_02_read) then
			avs_02_readdata <= pixel_02;
		end if;
	end process;
	
read_10: process(clk,avs_10_read) is
	begin
		if rising_edge(avs_10_read) then
			avs_10_readdata <= pixel_10;
		end if;
	end process;
	
read_11: process(clk,avs_11_read) is
	begin
		if rising_edge(avs_11_read) then
			avs_11_readdata <= pixel_11;
		end if;
	end process;
	
read_12: process(clk,avs_12_read) is
	begin
		if rising_edge(avs_12_read) then
			avs_12_readdata <= pixel_12;
		end if;
	end process;
	
read_20: process(clk,avs_20_read) is
	begin
		if rising_edge(avs_20_read) then
			avs_20_readdata <= pixel_20;
		end if;
	end process;
	
read_21: process(clk,avs_21_read) is
	begin
		if rising_edge(avs_21_read) then
			avs_21_readdata <= pixel_21;
		end if;
	end process;
	
read_22: process(clk,avs_22_read) is
	begin
		if rising_edge(avs_22_read) then
			avs_22_readdata <= pixel_22;
		end if;
	end process;
	
write_range: process(clk,avs_range_write) is
	begin
		if rising_edge(avs_range_write) then
			threshold_data_in <= avs_range_writedata;
		end if;
	end process;
	
	-- Output Registers
	PROCESS (clk)
	BEGIN
		IF clk'EVENT AND clk = '1' THEN
			IF (reset = '1') THEN
				stream_out_data			<= (OTHERS => '0');
				stream_out_startofpacket	<= '0';
				stream_out_endofpacket		<= '0';
				stream_out_valid				<= '0';
			ELSIF (transfer_data = '1') THEN
				IF (show_thresh = '1') THEN
					IF(pass_val = '1') THEN
						stream_out_data <= B"111111111111111111111111";
					ELSE
						stream_out_data <= B"000000000000000000000000";
					END IF;
				ELSE
					stream_out_data				<= stream_in_data;
				END IF;
				stream_out_startofpacket					<= stream_in_startofpacket;
				stream_out_endofpacket						<= stream_in_endofpacket;
				stream_out_valid								<= stream_in_valid;
			ELSIF ((stream_out_ready = '1') AND (stream_out_valid = '1')) THEN
				stream_out_data				<=  (OTHERS => '0');
				stream_out_startofpacket	<= '0';
				stream_out_endofpacket		<= '0';
				stream_out_valid				<= '0';
			END IF;
		END IF;
	END PROCESS;

	PROCESS (clk)
	BEGIN
		IF clk'EVENT AND clk = '1' THEN
			IF (reset = '1') THEN
				data								<=	(OTHERS => '0');
				startofpacket					<= '0';
				endofpacket						<= '0';
				valid								<= '0';
			ELSIF (stream_in_ready = '1') THEN
				data								<= stream_in_data;
				startofpacket					<= stream_in_startofpacket;
				endofpacket						<= stream_in_endofpacket;
				valid								<= stream_in_valid;
			ELSIF (transfer_data = '1') THEN
				data								<=  (OTHERS => '0');
				startofpacket					<= '0';
				endofpacket						<= '0';
				valid								<= '0';
			END IF;
		END IF;
	END PROCESS;
	
	stream_in_ready <= stream_in_valid AND ( NOT valid OR transfer_data);
	transfer_data <= NOT stream_out_valid AND stream_in_valid;
	
	rgv_to_hsv: RGB_to_HSV_converter
	PORT MAP (
		clk									=> clk,
		clk_en								=> transfer_data,
		reset								=> reset,
	
		R									=> data(23 DOWNTO  16),
		G									=> data(15 DOWNTO 8),
		B									=> data(7 DOWNTO  0),
		
		H									=> H_thresh (8 DOWNTO 0),
		S									=> S_thresh (7 DOWNTO 0),
		V									=> V_thresh (7 DOWNTO 0),
		
		H_range								=> H_thresh(31 DOWNTO 24),
		S_range								=> S_thresh(31 DOWNTO 24),
		V_range								=> V_thresh(31 DOWNTO 24),
				
		thershould_range					=> threshold_data_in,		
		stream_in_startofpacket				=> startofpacket,
		location_x							=> location(9 DOWNTO 0),
		location_y							=> location(19 DOWNTO 10),
		temp1								=> temp1_result,
		temp2								=> temp2_result,
		temp3								=> temp3_result,
		temp4								=> temp4_result,
		sample_pixel_00						=> pixel_00,
		sample_pixel_01						=> pixel_01,
		sample_pixel_02						=> pixel_02,
		sample_pixel_10						=> pixel_10,
		sample_pixel_11						=> pixel_11,
		sample_pixel_12						=> pixel_12,
		sample_pixel_20						=> pixel_20,
		sample_pixel_21						=> pixel_21,
		sample_pixel_22						=> pixel_22,
		stream_out_thresh					=> pass_val

	);
	
	END Behaviour;